from typing import Any

from memorix_client_redis.features.api.json import to_json


def hash_key(id: str, key: Any) -> str:
    if key is None:
        return to_json(value=[id])
    return to_json(
        value=[id, key],
        sort_dict=True,
    )



import typing


class CacheSetOptionsExpire(object):
    def __init__(
        self,
        value: int,
        is_in_ms: typing.Optional[bool] = False,
    ) -> None:
        self.value = value
        self.is_in_ms = is_in_ms

    @staticmethod
    def merge(
        item1: typing.Optional["CacheSetOptionsExpire"],
        item2: typing.Optional["CacheSetOptionsExpire"],
    ) -> typing.Optional["CacheSetOptionsExpire"]:
        if item2 is None:
            return item1
        return item2


class CacheSetOptions(object):
    Expire = CacheSetOptionsExpire

    def __init__(
        self,
        expire: typing.Optional[CacheSetOptionsExpire] = None,
    ) -> None:
        self.expire = expire

    @staticmethod
    def merge(
        item1: typing.Optional["CacheSetOptions"],
        item2: typing.Optional["CacheSetOptions"],
    ) -> typing.Optional["CacheSetOptions"]:
        if item1 is None:
            return item2
        if item2 is None:
            return item1
        return CacheSetOptions(
            expire=CacheSetOptions.Expire.merge(item1.expire, item2.expire),
        )



from .features.api.api import (
    Api as MemorixClientApi,
    ApiDefaults as MemorixClientApiDefaults,
)
from .features.api.cache.cache_api import CacheApi as MemorixClientCacheApi
from .features.api.cache.cache_item import (
    CacheItem as MemorixClientCacheApiItem,
    CacheItemNoKey as MemorixClientCacheApiItemNoKey,
)
from .features.api.cache.cache_options import (
    CacheSetOptions as MemorixClientCacheSetOptions,
    CacheSetOptionsExpire as MemorixClientCacheSetOptionsExpire,
)
from .features.api.pubsub.pubsub_api import PubSubApi as MemorixClientPubSubApi
from .features.api.pubsub.pubsub_item import (
    PubSubItem as MemorixClientPubSubApiItem,
    PubSubItemNoKey as MemorixClientPubSubApiItemNoKey,
)
from .features.api.task.task_api import TaskApi as MemorixClientTaskApi
from .features.api.task.task_item import (
    TaskItem as MemorixClientTaskApiItem,
    TaskItemNoKey as MemorixClientTaskApiItemNoKey,
    TaskItemNoReturns as MemorixClientTaskApiItemNoReturns,
    TaskItemNoKeyNoReturns as MemorixClientTaskApiItemNoKeyNoReturns,
)
from .features.api.task.task_options import (
    TaskDequequeOptions as MemorixClientTaskDequequeOptions,
)
from dataclasses import dataclass



from redis import Redis
import typing
from .cache.cache_options import CacheSetOptions as _CacheSetOptions
from .task.task_options import TaskDequequeOptions as _TaskDequequeOptions


class ApiDefaults(object):
    CacheSetOptions = _CacheSetOptions
    TaskDequequeOptions = _TaskDequequeOptions

    def __init__(
        self,
        cache_set_options: typing.Optional[_CacheSetOptions] = None,
        task_dequeque_options: typing.Optional[_TaskDequequeOptions] = None,
    ) -> None:
        self.cache_set_options = cache_set_options
        self.task_dequeque_options = task_dequeque_options

    @staticmethod
    def merge(
        item1: typing.Optional["ApiDefaults"],
        item2: typing.Optional["ApiDefaults"],
    ) -> typing.Optional["ApiDefaults"]:
        if item1 is None:
            return item2
        if item2 is None:
            return item1
        return ApiDefaults(
            cache_set_options=ApiDefaults.CacheSetOptions.merge(
                item1.cache_set_options,
                item2.cache_set_options,
            ),
            task_dequeque_options=ApiDefaults.TaskDequequeOptions.merge(
                item1.task_dequeque_options,
                item2.task_dequeque_options,
            ),
        )


class _ApiConfigDefaultOptions(object):
    def __init__(
        self,
        cache: typing.Optional[_CacheSetOptions] = None,
        task: typing.Optional[_TaskDequequeOptions] = None,
    ) -> None:
        self.cache = cache
        self.task = task


class _ApiConfig(object):
    DefaultOptions = _ApiConfigDefaultOptions

    def __init__(
        self,
        default_options: _ApiConfigDefaultOptions,
    ) -> None:
        self.default_options = default_options


class Api(object):
    Config = _ApiConfig

    def __init__(
        self,
        redis_url: str,
        defaults: typing.Optional[ApiDefaults] = None,
    ) -> None:
        self._redis = Redis.from_url(redis_url)
        self._pubsub = self._redis.pubsub()
        self._defaults = defaults

    @staticmethod
    def from_config(config: _ApiConfig) -> typing.Type["Api"]:
        class ApiWithConfig(Api):
            def __init__(
                self,
                redis_url: str,
                defaults: typing.Optional[ApiDefaults] = None,
            ) -> None:
                merged_defaults = ApiDefaults.merge(
                    ApiDefaults(
                        cache_set_options=config.default_options.cache,
                        task_dequeque_options=config.default_options.task,
                    ),
                    defaults,
                )

                super().__init__(redis_url=redis_url, defaults=merged_defaults)

        return ApiWithConfig



# flake8: noqa
import typing

if typing.TYPE_CHECKING:
    from dataclasses import dataclass
else:
    from memorix_client_redis import dataclass

from enum import Enum
from memorix_client_redis import (
    MemorixClientApi,
    MemorixClientApiDefaults as _MemorixClientApiDefaults,
    MemorixClientCacheApi,
    MemorixClientCacheApiItem,
    MemorixClientCacheApiItemNoKey,
    MemorixClientCacheSetOptions as _MemorixClientCacheSetOptions,
    MemorixClientCacheSetOptionsExpire as _MemorixClientCacheSetOptionsExpire,
    MemorixClientPubSubApi,
    MemorixClientPubSubApiItem,
    MemorixClientPubSubApiItemNoKey,
    MemorixClientTaskApi,
    MemorixClientTaskApiItem,
    MemorixClientTaskApiItemNoKey,
    MemorixClientTaskApiItemNoReturns,
    MemorixClientTaskApiItemNoKeyNoReturns,
    MemorixClientTaskDequequeOptions as _MemorixClientTaskDequequeOptions,
)


MemorixClientApiDefaults = _MemorixClientApiDefaults
MemorixClientCacheSetOptions = _MemorixClientCacheSetOptions
MemorixClientCacheSetOptionsExpire = _MemorixClientCacheSetOptionsExpire
MemorixClientTaskDequequeOptions = _MemorixClientTaskDequequeOptions


class Animal(str, Enum):
    dog = "dog"
    cat = "cat"
    person = "person"


@dataclass
class User(object):
    name: str
    age: typing.Optional[int]


class MemorixCacheApi(MemorixClientCacheApi):
    def __init__(self, api: MemorixClientApi) -> None:
        super().__init__(api=api)

        self.favoriteAnimal = MemorixClientCacheApiItem[str, "Animal"](
            api=self._api,
            id="favoriteAnimal",
            payload_class=Animal,
        )
        self.user = MemorixClientCacheApiItem[str, "User"](
            api=self._api,
            id="user",
            payload_class=User,
        )


class MemorixPubSubApi(MemorixClientPubSubApi):
    def __init__(self, api: MemorixClientApi) -> None:
        super().__init__(api=api)

        self.message = MemorixClientPubSubApiItemNoKey[str](
            api=self._api,
            id="message",
            payload_class=str,
        )


class MemorixTaskApi(MemorixClientTaskApi):
    def __init__(self, api: MemorixClientApi) -> None:
        super().__init__(api=api)

        self.runAlgo = MemorixClientTaskApiItemNoKey[str, "Animal"](
            api=self._api,
            id="runAlgo",
            payload_class=str,
            returns_class=Animal,
        )


class MemorixApi(MemorixClientApi):
    def __init__(
        self,
        redis_url: str,
        defaults: typing.Optional[MemorixClientApiDefaults] = None,
    ) -> None:
        super().__init__(redis_url=redis_url, defaults=defaults)

        self.cache = MemorixCacheApi(self)
        self.pubsub = MemorixPubSubApi(self)
        self.task = MemorixTaskApi(self)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['memorix_client_redis',
 'memorix_client_redis.features',
 'memorix_client_redis.features.api',
 'memorix_client_redis.features.api.cache',
 'memorix_client_redis.features.api.pubsub',
 'memorix_client_redis.features.api.task']

package_data = \
{'': ['*']}

install_requires = \
['dacite>=1.6.0,<2.0.0', 'redis>=4.3.4,<5.0.0']

entry_points = \
{'console_scripts': ['start = memorix_client_redis.start:start']}

setup_kwargs = {
    'name': 'memorix-client-redis',
    'version': '1.2.0',
    'description': '',
    'long_description': 'None',
    'author': 'Yuval Saraf',
    'author_email': 'unimonkiez@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
